@@ -5,18 +5,27 @@ import re
 import shlex
 from collections import namedtuple
 from dataclasses import dataclass, field
+from enum import Enum
 from pathlib import Path
-from typing import Any, Optional, get_args, get_origin, Literal, TypeVar, Generic
+from typing import (
+    Union, Any, Optional, get_args, get_origin,
+    Literal, TypeVar, Generic, Callable
+)
+from uuid import UUID
 
-from .exceptions import ParamNotFoundError, PosParamsCountError, KeywordParamNotFoundError
+from .exceptions import (
+    PosParamsCountError,
+    KeywordParamNotFoundError,
+    KeywordParamMissingError)
 
-T = TypeVar('T', str, int, float, dt.date, dt.datetime, Path, dict, Literal[None], list)
+T = TypeVar('T', str, int, float, dt.date, dt.datetime, Path, dict, list)
 
 
 def convert_to_type(data_type: Any, value: str):
     """
     Converts `value` to `data_type`, if not possible raises the appropriate error
     """
+
     if data_type is Any or data_type is bool:
         return value
     elif data_type is str:
@@ -25,6 +34,8 @@ def convert_to_type(data_type: Any, value: str):
         return int(value)
     elif data_type is float:
         return float(value)
+    elif data_type is UUID:
+        return UUID(value)
     elif data_type is dt.date:
         return dt.date.fromisoformat(value)
     elif data_type is dt.datetime:
@@ -46,6 +57,8 @@ def convert_to_type(data_type: Any, value: str):
         list_type = get_args(data_type)
         return [convert_to_type(list_type[0] if list_type else str,
                                 x) for x in value.split(' ')]
+    elif issubclass(data_type, Enum):
+        return data_type[value].value
     else:
         raise NotImplementedError(f'No parser implemented for data type "{data_type}"')
 
@@ -100,7 +113,7 @@ def Option(
         default: Any,
         *keyword_args: str,
         help: str = None
-):
+) -> Any:
     return CommandOption(
         default=default,
         help=help,
@@ -182,10 +195,11 @@ def get_cmd_args(cmd: str, types: dict[str, Any]) -> tuple[list[str], dict[str,
     return positional_args, keyword_params
 
 
-def get_default_args(func):
+def get_default_args(func) -> list[CommandOption]:
     signature = inspect.signature(func)
-    return [v.default if v is not inspect.Parameter.empty else None
-            for v in signature.parameters.values()]
+    return [v.default
+            for v in signature.parameters.values()
+            if v is not inspect.Parameter.empty]
 
 
 def parse_input_args(args: tuple[Any, ...], commands: set[str]) -> tuple[
@@ -245,13 +259,38 @@ def convert_args_to_dict(input_args: list[str],
     for _keyword_arg_key, _keyword_arg_value in _input_keyword_args.items():
         _keyword_param = keyword_args.get(_keyword_arg_key)
         if not _keyword_param:
-            raise ParamNotFoundError(f'Could not find param {_keyword_arg_key}',
-                                     _keyword_arg_key)
+            raise KeywordParamMissingError(f'Missing value for required keyword parameter {_keyword_arg_key!r}',
+                                           _keyword_arg_key)
         fn_args[_keyword_param.name] = _keyword_param.validate(_keyword_arg_value)
 
-    # We fill optional fields with None
+    # We fill optional fields with None and check for missing ones
     for _arg in options:
         if _arg.name not in fn_args:
+            if _arg.is_required:
+                raise KeywordParamMissingError(f'Missing value for required keyword parameter {_arg.name!r}',
+                                               _arg.name)
             fn_args[_arg.name] = None if _arg.default is ... else _arg.default
 
     return fn_args
+
+
+@dataclass
+class CommandDerivedOption:
+    processor: Callable
+    param_name: Optional[str] = field(init=False)
+
+    def update_args(self, args: dict) -> dict:
+        if self.param_name is None:
+            raise ValueError('param_name not set. Did you forgot to set it?')
+        _args = args.copy()
+        fn_args = {}
+        for _opt in get_default_args(self.processor):
+            fn_args[_opt.name] = _args.pop(_opt.name)
+        _args[self.param_name] = self.processor(**fn_args)
+        return _args
+
+
+def Derived(
+        processor: Callable
+) -> Any:
+    return CommandDerivedOption(processor=processor)
